@@ -1,9 +1,9 @@
 /*
- *  $Id: guage.c,v 1.52 2011/01/17 10:39:28 tom Exp $
+ *  $Id: guage.c,v 1.69 2015/02/26 02:07:12 tom Exp $
  *
  *  guage.c -- implements the gauge dialog
  *
- *  Copyright 2000-2010,2011	Thomas E. Dickey
+ *  Copyright 2000-2013,2015	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -35,10 +35,11 @@
 
 #define isMarker(buf) !strncmp(buf, "XXX", (size_t) 3)
 
-typedef struct {
-    DIALOG_CALLBACK obj;
+typedef struct _my_obj {
+    DIALOG_CALLBACK obj;	/* has to be first in struct */
+    struct _my_obj *next;
     WINDOW *text;
-    const char *title;
+    char *title;
     char *prompt;
     char prompt_buf[MY_LEN];
     int percent;
@@ -47,6 +48,43 @@ typedef struct {
     char line[MAX_LEN + 1];
 } MY_OBJ;
 
+static MY_OBJ *all_objects;
+
+static int
+valid(MY_OBJ * obj)
+{
+    MY_OBJ *list = all_objects;
+    int result = 0;
+
+    while (list != 0) {
+	if (list == obj) {
+	    result = 1;
+	    break;
+	}
+	list = list->next;
+    }
+    return result;
+}
+
+static void
+delink(MY_OBJ * obj)
+{
+    MY_OBJ *p = all_objects;
+    MY_OBJ *q = 0;
+    while (p != 0) {
+	if (p == obj) {
+	    if (q != 0) {
+		q->next = p->next;
+	    } else {
+		all_objects = p->next;
+	    }
+	    break;
+	}
+	q = p;
+	p = p->next;
+    }
+}
+
 static int
 read_data(char *buffer, FILE *fp)
 {
@@ -55,6 +93,8 @@ read_data(char *buffer, FILE *fp)
     if (feof(fp)) {
 	result = 0;
     } else if (fgets(buffer, MY_LEN, fp) != 0) {
+	DLG_TRACE(("read_data:%s", buffer));
+	buffer[MY_LEN] = '\0';
 	dlg_trim_string(buffer);
 	result = 1;
     } else {
@@ -81,28 +121,31 @@ repaint_text(MY_OBJ * obj)
     WINDOW *dialog = obj->obj.win;
     int i, x;
 
-    if (dialog != 0 && obj->obj.input != 0) {
+    if (dialog != 0) {
 	(void) werase(dialog);
-	dlg_draw_box(dialog, 0, 0, obj->height, obj->width, dialog_attr, border_attr);
+	dlg_draw_box2(dialog, 0, 0, obj->height, obj->width, dialog_attr,
+		      border_attr, border2_attr);
 
 	dlg_draw_title(dialog, obj->title);
 
-	wattrset(dialog, dialog_attr);
+	(void) wattrset(dialog, dialog_attr);
+	dlg_draw_helpline(dialog, FALSE);
 	dlg_print_autowrap(dialog, obj->prompt, obj->height, obj->width);
 
-	dlg_draw_box(dialog,
-		     obj->height - 4, 2 + MARGIN,
-		     2 + MARGIN, obj->width - 2 * (2 + MARGIN),
-		     dialog_attr,
-		     border_attr);
+	dlg_draw_box2(dialog,
+		      obj->height - 4, 2 + MARGIN,
+		      2 + MARGIN, obj->width - 2 * (2 + MARGIN),
+		      dialog_attr,
+		      border_attr,
+		      border2_attr);
 
 	/*
 	 * Clear the area for the progress bar by filling it with spaces
-	 * in the title-attribute, and write the percentage with that
+	 * in the gauge-attribute, and write the percentage with that
 	 * attribute.
 	 */
 	(void) wmove(dialog, obj->height - 3, 4);
-	wattrset(dialog, gauge_attr);
+	(void) wattrset(dialog, gauge_attr);
 
 	for (i = 0; i < (obj->width - 2 * (3 + MARGIN)); i++)
 	    (void) waddch(dialog, ' ');
@@ -116,15 +159,15 @@ repaint_text(MY_OBJ * obj)
 	 * but requires some tweaks to reverse it.
 	 */
 	x = (obj->percent * (obj->width - 2 * (3 + MARGIN))) / 100;
-	if ((title_attr & A_REVERSE) != 0) {
+	if ((gauge_attr & A_REVERSE) != 0) {
 	    wattroff(dialog, A_REVERSE);
 	} else {
-	    wattrset(dialog, A_REVERSE);
+	    (void) wattrset(dialog, A_REVERSE);
 	}
 	(void) wmove(dialog, obj->height - 3, 4);
 	for (i = 0; i < x; i++) {
 	    chtype ch2 = winch(dialog);
-	    if (title_attr & A_REVERSE) {
+	    if (gauge_attr & A_REVERSE) {
 		ch2 &= ~A_REVERSE;
 	    }
 	    (void) waddch(dialog, ch2);
@@ -140,10 +183,12 @@ handle_input(DIALOG_CALLBACK * cb)
     MY_OBJ *obj = (MY_OBJ *) cb;
     bool result;
     int status;
-    char buf[MY_LEN];
+    char buf[MY_LEN + 1];
 
     if (dialog_state.pipe_input == 0) {
 	status = -1;
+	delink(obj);
+	dlg_remove_callback(cb);
     } else if ((status = read_data(buf, dialog_state.pipe_input)) > 0) {
 
 	if (isMarker(buf)) {
@@ -179,6 +224,7 @@ handle_input(DIALOG_CALLBACK * cb)
     } else {
 	if (feof(dialog_state.pipe_input) ||
 	    (ferror(dialog_state.pipe_input) && errno != EINTR)) {
+	    delink(obj);
 	    dlg_remove_callback(cb);
 	}
     }
@@ -187,6 +233,7 @@ handle_input(DIALOG_CALLBACK * cb)
 	result = TRUE;
 	repaint_text(obj);
     } else {
+	repaint_text(obj);
 	result = FALSE;
     }
 
@@ -202,7 +249,8 @@ handle_my_getc(DIALOG_CALLBACK * cb, int ch, int fkey, int *result)
     if (cb != 0) {
 	if (!fkey && (ch == ERR)) {
 	    (void) handle_input(cb);
-	    status = (cb->input != 0);
+	    /* cb might be freed in handle_input */
+	    status = (valid((MY_OBJ *) cb) && (cb->input != 0));
 	}
     } else {
 	status = FALSE;
@@ -215,9 +263,120 @@ my_cleanup(DIALOG_CALLBACK * cb)
 {
     MY_OBJ *obj = (MY_OBJ *) cb;
 
-    if (obj != 0) {
-	if (obj->prompt != obj->prompt_buf)
+    if (valid(obj)) {
+	if (obj->prompt != obj->prompt_buf) {
 	    free(obj->prompt);
+	    obj->prompt = obj->prompt_buf;
+	}
+	delink(obj);
+    }
+}
+
+void
+dlg_update_gauge(void *objptr, int percent)
+{
+    MY_OBJ *obj = (MY_OBJ *) objptr;
+    bool save_finish_string = dialog_state.finish_string;
+
+    dialog_state.finish_string = TRUE;
+    curs_set(0);
+    obj->percent = percent;
+    repaint_text(obj);
+    dialog_state.finish_string = save_finish_string;
+}
+
+/*
+ * (Re)Allocates an object and fills it as per the arguments
+ */
+void *
+dlg_reallocate_gauge(void *objptr,
+		     const char *title,
+		     const char *cprompt,
+		     int height,
+		     int width,
+		     int percent)
+{
+    char *prompt = dlg_strclone(cprompt);
+    MY_OBJ *obj = objptr;
+    bool save_finish_string = dialog_state.finish_string;
+
+    dialog_state.finish_string = TRUE;
+    dlg_tab_correct_str(prompt);
+
+    if (objptr == 0) {
+	/* create a new object */
+	obj = dlg_calloc(MY_OBJ, 1);
+	assert_ptr(obj, "dialog_gauge");
+
+	dlg_auto_size(title, prompt, &height, &width, MIN_HIGH, MIN_WIDE);
+	dlg_print_size(height, width);
+	dlg_ctl_size(height, width);
+
+    } else {
+	/* reuse an existing object */
+	obj = objptr;
+	height = obj->height;
+	width = obj->width;
+    }
+
+    if (obj->obj.win == 0) {
+	/* center dialog box on screen */
+	int x = dlg_box_x_ordinate(width);
+	int y = dlg_box_y_ordinate(height);
+	WINDOW *dialog = dlg_new_window(height, width, y, x);
+	obj->obj.win = dialog;
+    }
+
+    obj->obj.input = dialog_state.pipe_input;
+    obj->obj.keep_win = TRUE;
+    obj->obj.bg_task = TRUE;
+    obj->obj.handle_getc = handle_my_getc;
+    obj->obj.handle_input = handle_input;
+
+    if (obj->title == 0 || strcmp(obj->title, title)) {
+	dlg_finish_string(obj->title);
+	free(obj->title);
+	obj->title = dlg_strclone(title);
+    }
+
+    dlg_finish_string(obj->prompt);
+    free(obj->prompt);
+
+    obj->prompt = prompt;
+    obj->percent = percent;
+    obj->height = height;
+    obj->width = width;
+
+    /* if this was a new object, link it into the list */
+    if (objptr == 0) {
+	obj->next = all_objects;
+	all_objects = obj;
+    }
+
+    dialog_state.finish_string = save_finish_string;
+    return (void *) obj;
+}
+
+void *
+dlg_allocate_gauge(const char *title,
+		   const char *cprompt,
+		   int height,
+		   int width,
+		   int percent)
+{
+    return dlg_reallocate_gauge(NULL, title, cprompt, height, width, percent);
+}
+
+void
+dlg_free_gauge(void *objptr)
+{
+    MY_OBJ *obj = (MY_OBJ *) objptr;
+
+    curs_set(1);
+    if (valid(obj)) {
+	delink(obj);
+	obj->obj.keep_win = FALSE;
+	dlg_remove_callback(&(obj->obj));
     }
 }
 
@@ -236,80 +395,47 @@ dialog_gauge(const char *title,
 	     int width,
 	     int percent)
 {
-#ifdef KEY_RESIZE
-    int old_height = height;
-    int old_width = width;
-#endif
     int fkey;
     int ch, result;
-    int x, y;
-    char *prompt = dlg_strclone(cprompt);
-    WINDOW *dialog;
-    MY_OBJ *obj = 0;
+    void *objptr = dlg_allocate_gauge(title, cprompt, height, width, percent);
+    MY_OBJ *obj = (MY_OBJ *) objptr;
 
-    curs_set(0);
-
-    dlg_tab_correct_str(prompt);
-
-#ifdef KEY_RESIZE
-  retry:
-#endif
-
-    dlg_auto_size(title, prompt, &height, &width, MIN_HIGH, MIN_WIDE);
-    dlg_print_size(height, width);
-    dlg_ctl_size(height, width);
-
-    /* center dialog box on screen */
-    x = dlg_box_x_ordinate(width);
-    y = dlg_box_y_ordinate(height);
-
-    dialog = dlg_new_window(height, width, y, x);
-
-    if (obj == 0) {
-	MY_OBJ **objref;
-
-	obj = dlg_calloc(MY_OBJ, 1);
-	assert_ptr(obj, "dialog_gauge");
-
-	objref = &obj;
-	obj->obj.input = dialog_state.pipe_input;
-	obj->obj.win = dialog;
-	obj->obj.keep_win = TRUE;
-	obj->obj.bg_task = TRUE;
-	obj->obj.handle_getc = handle_my_getc;
-	obj->obj.handle_input = handle_input;
-	obj->title = title;
-	obj->prompt = prompt;
-	obj->percent = percent;
-	obj->height = height;
-	obj->width = width;
-	dlg_add_callback_ref((DIALOG_CALLBACK **) objref, my_cleanup);
-    } else {
-	obj->obj.win = dialog;
-    }
-
-    repaint_text(obj);
+    dlg_add_callback_ref((DIALOG_CALLBACK **) & obj, my_cleanup);
+    dlg_update_gauge(obj, percent);
 
+    dlg_trace_win(obj->obj.win);
     do {
-	ch = dlg_getc(dialog, &fkey);
+	ch = dlg_getc(obj->obj.win, &fkey);
 #ifdef KEY_RESIZE
 	if (fkey && ch == KEY_RESIZE) {
-	    /* reset data */
-	    height = old_height;
-	    width = old_width;
-	    /* repaint */
-	    dlg_clear();
-	    dlg_del_window(dialog);
-	    refresh();
+	    MY_OBJ *oldobj = obj;
+
 	    dlg_mouse_free_regions();
-	    goto retry;
+
+	    obj = dlg_allocate_gauge(title,
+				     cprompt,
+				     height,
+				     width,
+				     oldobj->percent);
+
+	    /* avoid breaking new window in dlg_remove_callback */
+	    oldobj->obj.caller = 0;
+	    oldobj->obj.input = 0;
+	    oldobj->obj.keep_win = FALSE;
+
+	    /* remove the old version of the gauge */
+	    dlg_clear();
+	    dlg_remove_callback(&(oldobj->obj));
+	    refresh();
+
+	    dlg_add_callback_ref((DIALOG_CALLBACK **) & obj, my_cleanup);
+	    dlg_update_gauge(obj, obj->percent);
 	}
 #endif
     }
-    while (handle_my_getc(&(obj->obj), ch, fkey, &result));
+    while (valid(obj) && handle_my_getc(&(obj->obj), ch, fkey, &result));
 
-    curs_set(1);
-    dlg_del_window(dialog);
+    dlg_free_gauge(obj);
 
     return (DLG_EXIT_OK);
 }
